unit MyMath;

{$mode objfpc}{$H+}
{$modeswitch advancedrecords}

interface

uses
  Classes, SysUtils, Types, Math, Dialogs;

type

  //TPointf = record
  //  x, y: single;
  //end;

  { TSchubKurbel }

  TSchubKurbel = record
    Stange1_W, Stange2_W: single;
    procedure SetParam(Point1, Point2: TPointF; Stange1_L, Stange2_L: single; reverse: boolean = False);
  end;

  { TKurbelSchwinge }

  TKurbelSchwinge = record
    Pleuel_W, Schwinge_W: single;
    RadExzenter: TPointF;
    procedure SetParam(Rad_W, Rad_R: single; Schwinge_xy: TPointF; Pleuel_L, Schwinge_L: single; reverse: boolean = False);
  end;

  { TTrigo }

  TTrigo = record
    Grad: boolean;
    function a_b_c_To_Alpha(a, b, c: single): single;
    function a_b_c_To_Beta(a, b, c: single): single;
    function a_b_c_To_Gama(a, b, c: single): single;

    // Dreieck rechtwinklig

    function Alpha_c_To_xy(angle, len: single): TPointf;

    function a_c_To_Alpha(a, c: single): single;
    function a_c_To_Beta(a, c: single): single;
    function b_c_To_Alpha(b, c: single): single;
    function b_c_To_Beta(b, c: single): single;

    function a_b_To_Alpha(a, b: single): single;
    function a_b_To_Beta(a, b: single): single;

    function a_b_To_c(a, b: single): single;
  end;

var
  Trigo: TTrigo;

function PointF(x, y: single): TPointF;


implementation

function PointF(x, y: single): TPointF;
begin
  Result.x := x;
  Result.y := y;
end;

{ TTrigo }

function TTrigo.a_b_c_To_Alpha(a, b, c: single): single;
begin
  Result := arccos((-sqr(a) + sqr(b) + sqr(c)) / (2 * b * c));
  if Grad then begin
    Result := Result / pi * 180;
  end;
end;

function TTrigo.a_b_c_To_Beta(a, b, c: single): single;
begin
  Result := arccos((-sqr(b) + sqr(a) + sqr(c)) / (2 * a * c));
  if Grad then begin
    Result := Result / pi * 180;
  end;
end;

function TTrigo.a_b_c_To_Gama(a, b, c: single): single;
begin
  Result := arccos((-sqr(c) + sqr(a) + sqr(b)) / (2 * a * b));
  if Grad then begin
    Result := Result / pi * 180;
  end;
end;

function TTrigo.Alpha_c_To_xy(angle, len: single): TPointf;
begin
  if Grad then begin
    angle := angle / 180.0 * Pi;
  end;
  Result.x := cos(angle) * len;
  Result.y := sin(angle) * len;
end;

function TTrigo.a_c_To_Alpha(a, c: single): single;
begin
  Result := arcsin(a / c);
  if grad then begin
    Result := Result / pi * 180;
  end;
end;

function TTrigo.a_c_To_Beta(a, c: single): single;
begin
  Result := arccos(a / c);
  if grad then begin
    Result := Result / pi * 180;
  end;
end;

function TTrigo.b_c_To_Alpha(b, c: single): single;
begin
  Result := arccos(b / c);
  if grad then begin
    Result := Result / pi * 180;
  end;
end;

function TTrigo.b_c_To_Beta(b, c: single): single;
begin
  Result := arcsin(b / c);
  if grad then begin
    Result := Result / pi * 180;
  end;
end;

function TTrigo.a_b_To_Alpha(a, b: single): single;
begin
  Result := arctan(a / b);
  if grad then begin
    Result := Result / pi * 180;
  end;
end;

function TTrigo.a_b_To_Beta(a, b: single): single;
begin
  Result := arctan(b / a);
  if grad then begin
    Result := Result / pi * 180;
  end;
end;

function TTrigo.a_b_To_c(a, b: single): single;
begin
  Result := sqrt(sqr(a) + sqr(b));
end;


{ TSchubKurbel }

procedure TSchubKurbel.SetParam(Point1, Point2: TPointF; Stange1_L, Stange2_L: single; reverse: boolean);
var
  c,             // Ist der Abstand zwischen dem Radexcenter und dem Zentrum der Schwinge_xy.
  w_c: single;   // Ist der Winkel von C   ( Ister der Winkel von c, wen die eine Linie wäre.
begin
  c := Trigo.a_b_To_c(Point1.y - Point2.y, Point2.x - Point1.x);

  if Point1.x < Point2.x then begin
    w_c := Trigo.a_c_To_Alpha(Point2.y - Point1.y, c);
  end else begin
    w_c := Trigo.a_c_To_Alpha(Point1.y - Point2.y, c) - 180;
  end;

  if reverse then begin
    Stange1_W := w_c - Trigo.a_b_c_To_Beta(Stange1_L, Stange2_L, c);
    Stange2_W := w_c + Trigo.a_b_c_To_Alpha(Stange1_L, Stange2_L, c) + 180;
  end else begin
    Stange1_W := w_c + Trigo.a_b_c_To_Alpha(Stange2_L, Stange1_L, c);
    Stange2_W := w_c - Trigo.a_b_c_To_Beta(Stange2_L, Stange1_L, c) + 180;
  end;

end;

{ TKurbelSchwinge }

procedure TKurbelSchwinge.SetParam(Rad_W, Rad_R: single; Schwinge_xy: TPointF; Pleuel_L, Schwinge_L: single; reverse: boolean);
(*  @Rad_W:           Winkel, des Excenter, des Rades.
    @Rad_R:           Distanz, Rad-Zentrum bis Excenter.
    @Schwinge_x / y:  Drehpunkt der Schwinge.
    @Pleuel_L:        Länge des Pleuels, vom Rad-exenter bis Schwinge-Ende, im Beispiel Verbindung der blauen und grünen Stange.
    @Schwinge_L:      Distanz, Drehpunkt-Schwinge bis Pleuelbefestigung.
    @reverse:         Rechnet seitenverkehrt.
*)

var
  KnieHbl: TSchubKurbel;
begin
  RadExzenter := Trigo.Alpha_c_To_xy(Rad_W, Rad_R);

  KnieHbl.SetParam(RadExzenter, Schwinge_xy, Pleuel_L, Schwinge_L, reverse);
  Pleuel_W := KnieHbl.Stange1_W;
  Schwinge_W := KnieHbl.Stange2_W;
end;


end.
